@@ -15,7 +15,7 @@ import (
 
 	"github.com/percona/percona-backup-mongodb/pbm"
 	"github.com/percona/percona-backup-mongodb/pbm/compress"
-	plog "github.com/percona/percona-backup-mongodb/pbm/log"
+	"github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/version"
 )
 
@@ -44,7 +44,9 @@ type logsOpts struct {
 	severity string
 	event    string
 	opid     string
+	location string
 	extr     bool
+	follow   bool
 }
 
 type cliResult interface {
@@ -136,11 +138,13 @@ func Main() {
 
 	logsCmd := pbmCmd.Command("logs", "PBM logs")
 	logs := logsOpts{}
+	logsCmd.Flag("follow", "Follow output").Short('f').Default("false").BoolVar(&logs.follow)
 	logsCmd.Flag("tail", "Show last N entries, 20 entries are shown by default, 0 for all logs").Short('t').Default("20").Int64Var(&logs.tail)
 	logsCmd.Flag("node", "Target node in format replset[/host:posrt]").Short('n').StringVar(&logs.node)
 	logsCmd.Flag("severity", "Severity level D, I, W, E or F, low to high. Choosing one includes higher levels too.").Short('s').Default("I").EnumVar(&logs.severity, "D", "I", "W", "E", "F")
 	logsCmd.Flag("event", "Event in format backup[/2020-10-06T11:45:14Z]. Events: backup, restore, cancelBackup, resync, pitr, pitrestore, delete").Short('e').StringVar(&logs.event)
 	logsCmd.Flag("opid", "Operation ID").Short('i').StringVar(&logs.opid)
+	logsCmd.Flag("timezone", "Timezone of log output. `Local`, `UTC` or a location name corresponding to a file in the IANA Time Zone database, such as `America/New_York`").StringVar(&logs.location)
 	logsCmd.Flag("extra", "Show extra data in text format").Hidden().Short('x').BoolVar(&logs.extr)
 
 	statusOpts := statusOptions{}
@@ -283,7 +287,7 @@ func exitErr(e error, f outFormat) {
 }
 
 func runLogs(cn *pbm.PBM, l *logsOpts) (fmt.Stringer, error) {
-	r := &plog.LogRequest{}
+	r := &log.LogRequest{}
 
 	if l.node != "" {
 		n := strings.Split(l.node, "/")
@@ -307,17 +311,22 @@ func runLogs(cn *pbm.PBM, l *logsOpts) (fmt.Stringer, error) {
 
 	switch l.severity {
 	case "F":
-		r.Severity = plog.Fatal
+		r.Severity = log.Fatal
 	case "E":
-		r.Severity = plog.Error
+		r.Severity = log.Error
 	case "W":
-		r.Severity = plog.Warning
+		r.Severity = log.Warning
 	case "I":
-		r.Severity = plog.Info
+		r.Severity = log.Info
 	case "D":
-		r.Severity = plog.Debug
+		r.Severity = log.Debug
 	default:
-		r.Severity = plog.Info
+		r.Severity = log.Info
+	}
+
+	if l.follow {
+		err := followLogs(cn, r, r.Node == "", l.extr)
+		return nil, err
 	}
 
 	o, err := cn.LogGet(r, l.tail)
@@ -334,9 +343,39 @@ func runLogs(cn *pbm.PBM, l *logsOpts) (fmt.Stringer, error) {
 		o.Data[i], o.Data[opp] = o.Data[opp], o.Data[i]
 	}
 
+	err = o.SetLocation(l.location)
+	if err != nil {
+		fmt.Fprintf(os.Stderr, "Error: failed to parse timezone: %v\n\n", err)
+	}
+
 	return o, nil
 }
 
+func followLogs(cn *pbm.PBM, r *log.LogRequest, showNode, expr bool) error {
+	outC, errC := log.Follow(cn.Context(), cn.Conn.Database(pbm.DB).Collection(pbm.LogCollection), r, false)
+
+	for {
+		select {
+		case entry, ok := <-outC:
+			if !ok {
+				return nil
+			}
+
+			fmt.Println(entry.Stringify(tsUTC, showNode, expr))
+		case err, ok := <-errC:
+			if !ok {
+				return nil
+			}
+
+			return err
+		}
+	}
+}
+
+func tsUTC(ts int64) string {
+	return time.Unix(ts, 0).UTC().Format(time.RFC3339)
+}
+
 type snapshotStat struct {
 	Name       string         `json:"name"`
 	Namespaces []string       `json:"nss,omitempty"`
@@ -496,9 +535,9 @@ func isTTY() bool {
 
 func lastLogErr(cn *pbm.PBM, op pbm.Command, after int64) (string, error) {
 	l, err := cn.LogGet(
-		&plog.LogRequest{
-			LogKeys: plog.LogKeys{
-				Severity: plog.Error,
+		&log.LogRequest{
+			LogKeys: log.LogKeys{
+				Severity: log.Error,
 				Event:    string(op),
 			},
 		}, 1)
