@@ -5,14 +5,18 @@ import (
 	"context"
 	"encoding/json"
 	"io"
+	"strings"
 	"time"
 
 	mlog "github.com/mongodb/mongo-tools/common/log"
 	"github.com/pkg/errors"
+	"go.mongodb.org/mongo-driver/bson"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/mongo"
+	"go.mongodb.org/mongo-driver/mongo/options"
 
 	"github.com/percona/percona-backup-mongodb/pbm"
+	"github.com/percona/percona-backup-mongodb/pbm/compress"
 	plog "github.com/percona/percona-backup-mongodb/pbm/log"
 	"github.com/percona/percona-backup-mongodb/pbm/storage"
 	"github.com/percona/percona-backup-mongodb/version"
@@ -48,7 +52,7 @@ func NewPhysical(cn *pbm.PBM, node *pbm.Node) *Backup {
 	}
 }
 
-func (b *Backup) Init(bcp pbm.BackupCmd, opid pbm.OPID, balancer pbm.BalancerMode) error {
+func (b *Backup) Init(bcp *pbm.BackupCmd, opid pbm.OPID, balancer pbm.BalancerMode) error {
 	ts, err := b.cn.ClusterTime()
 	if err != nil {
 		return errors.Wrap(err, "read cluster time")
@@ -58,6 +62,7 @@ func (b *Backup) Init(bcp pbm.BackupCmd, opid pbm.OPID, balancer pbm.BalancerMod
 		Type:           b.typ,
 		OPID:           opid.String(),
 		Name:           bcp.Name,
+		Namespaces:     bcp.Namespaces,
 		Compression:    bcp.Compression,
 		StartTS:        time.Now().Unix(),
 		Status:         pbm.StatusStarting,
@@ -88,7 +93,7 @@ func (b *Backup) Init(bcp pbm.BackupCmd, opid pbm.OPID, balancer pbm.BalancerMod
 
 // Run runs backup.
 // TODO: describe flow
-func (b *Backup) Run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
+func (b *Backup) Run(ctx context.Context, bcp *pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
 	inf, err := b.node.GetInfo()
 	if err != nil {
 		return errors.Wrap(err, "get cluster info")
@@ -101,6 +106,9 @@ func (b *Backup) Run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 		Conditions:   []pbm.Condition{},
 		FirstWriteTS: primitive.Timestamp{T: 1, I: 1},
 	}
+	if v := inf.IsConfigSrv(); v {
+		rsMeta.IsConfigSvr = &v
+	}
 
 	stg, err := b.cn.GetStorage(l)
 	if err != nil {
@@ -118,14 +126,6 @@ func (b *Backup) Run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 			status := pbm.StatusError
 			if errors.Is(err, ErrCancelled) {
 				status = pbm.StatusCancelled
-
-				meta := &pbm.BackupMeta{
-					Name:     bcp.Name,
-					Status:   pbm.StatusCancelled,
-					Replsets: []pbm.BackupReplset{rsMeta},
-				}
-
-				l.Info("delete artifacts from storage: %v", b.cn.DeleteBackupFiles(meta, stg))
 			}
 
 			ferr := b.cn.ChangeRSState(bcp.Name, rsMeta.Name, status, err.Error())
@@ -178,6 +178,13 @@ func (b *Backup) Run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 			}
 		}()
 
+		if inf.IsConfigSrv() && len(bcp.Namespaces) != 0 {
+			err := checkNamespaceForBackup(ctx, b.cn.Conn, bcp.Namespaces[0])
+			if err != nil {
+				return errors.WithMessage(err, "namespace check")
+			}
+		}
+
 		if bcpm.BalancerStatus == pbm.BalancerModeOn {
 			err = b.cn.SetBalancerStatus(pbm.BalancerModeOff)
 			if err != nil {
@@ -196,6 +203,16 @@ func (b *Backup) Run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 		return errors.Wrap(err, "waiting for start")
 	}
 
+	defer func() {
+		if !errors.Is(err, ErrCancelled) || !inf.IsLeader() {
+			return
+		}
+
+		if err := b.cn.DeleteBackupFiles(bcpm, stg); err != nil {
+			l.Error("Failed to delete leftover files for canceled backup %q", bcpm.Name)
+		}
+	}()
+
 	switch b.typ {
 	case pbm.LogicalBackup:
 		err = b.doLogical(ctx, bcp, opid, &rsMeta, inf, stg, l)
@@ -226,7 +243,12 @@ func (b *Backup) Run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 			return errors.Wrap(err, "check cluster for backup done")
 		}
 
-		err = b.dumpClusterMeta(bcp.Name, stg)
+		bcpm, err = b.cn.GetBackupMeta(bcp.Name)
+		if err != nil {
+			return errors.Wrap(err, "get backup metadata")
+		}
+
+		err = writeMeta(stg, bcpm)
 		if err != nil {
 			return errors.Wrap(err, "dump metadata")
 		}
@@ -237,6 +259,55 @@ func (b *Backup) Run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 	return errors.Wrap(err, "waiting for done")
 }
 
+func checkNamespaceForBackup(ctx context.Context, m *mongo.Client, ns string) error {
+	db, coll := parseNS(ns)
+
+	res, err := getShardedNamespaces(ctx, m, db, coll)
+	if err != nil {
+		return errors.WithMessage(err, "get sharded namespaces")
+	}
+	if len(res) != 0 {
+		return errors.Errorf("selective backup: sharded collections: %s", strings.Join(res, ", "))
+	}
+
+	return nil
+}
+
+func getShardedNamespaces(ctx context.Context, m *mongo.Client, db, coll string) ([]string, error) {
+	cur, err := m.Database("config").Collection("collections").
+		Find(ctx, bson.D{}, options.Find().SetProjection(bson.D{{"_id", 1}}))
+	if err != nil {
+		return nil, errors.WithMessage(err, "query")
+	}
+	defer cur.Close(context.Background())
+
+	type document struct {
+		ID string `bson:"_id"`
+	}
+
+	rv := []string{}
+	for cur.Next(ctx) {
+		var doc document
+		if err := cur.Decode(&doc); err != nil {
+			return nil, errors.WithMessage(err, "decode")
+		}
+
+		d, c, _ := strings.Cut(doc.ID, ".")
+		if db != "" && db != d {
+			continue
+		}
+
+		c = strings.TrimPrefix(c, "system.buckets.")
+		if coll != "" && coll != c {
+			continue
+		}
+
+		rv = append(rv, d+"."+c)
+	}
+
+	return rv, nil
+}
+
 func waitForBalancerOff(cn *pbm.PBM, t time.Duration, l *plog.Event) pbm.BalancerMode {
 	dn := time.NewTimer(t)
 	defer dn.Stop()
@@ -325,10 +396,10 @@ type Canceller interface {
 var ErrCancelled = errors.New("backup canceled")
 
 // Upload writes data to dst from given src and returns an amount of written bytes
-func Upload(ctx context.Context, src Source, dst storage.Storage, compression pbm.CompressionType, compressLevel *int, fname string, sizeb int) (int64, error) {
+func Upload(ctx context.Context, src Source, dst storage.Storage, compression compress.CompressionType, compressLevel *int, fname string, sizeb int64) (int64, error) {
 	r, pw := io.Pipe()
 
-	w, err := Compress(pw, compression, compressLevel)
+	w, err := compress.Compress(pw, compression, compressLevel)
 	if err != nil {
 		return 0, err
 	}
@@ -559,15 +630,6 @@ func (b *Backup) waitForFirstLastWrite(bcpName string) (first, last primitive.Ti
 	}
 }
 
-func (b *Backup) dumpClusterMeta(bcpName string, stg storage.Storage) error {
-	meta, err := b.cn.GetBackupMeta(bcpName)
-	if err != nil {
-		return errors.Wrap(err, "get backup metadata")
-	}
-
-	return writeMeta(stg, meta)
-}
-
 func writeMeta(stg storage.Storage, meta *pbm.BackupMeta) error {
 	b, err := json.MarshalIndent(meta, "", "\t")
 	if err != nil {
